@@ -53,7 +53,7 @@
  * ## Dialog Integration
  *
  * When placed inside an [class@Dialog], `AdwHeaderBar` will display the dialog
- * title intead of window title. It will also adjust the decoration layout to
+ * title instead of window title. It will also adjust the decoration layout to
  * ensure it always has a close button and nothing else. Set
  * [property@HeaderBar:show-start-title-buttons] and
  * [property@HeaderBar:show-end-title-buttons] to `FALSE` to remove it if it's
@@ -78,6 +78,13 @@
  * `AdwHeaderBar` will automatically hide the title buttons other than at the
  * edges of the window.
  *
+ * ## Bottom Sheet Integration
+ *
+ * When played inside [class@BottomSheet], `AdwHeaderBar` will not show the title
+ * unless [property@BottomSheet:show-drag-handle] is set to `FALSE`, regardless
+ * of [property@HeaderBar:show-title]. This only applies to the default title,
+ * titles set with [property@HeaderBar:title-widget] will still be shown.
+ *
  * ## Centering Policy
  *
  * [property@HeaderBar:centering-policy] allows to enforce strict centering of
@@ -241,10 +248,21 @@ update_decoration_layout (AdwHeaderBar *self,
 }
 
 static void
-create_start_controls (AdwHeaderBar *self)
+recreate_start_controls (AdwHeaderBar *self)
 {
   GtkWidget *controls;
 
+  if (self->start_controls) {
+    if (self->sheet && ADW_IS_SHEET_CONTROLS (self->start_controls))
+      return;
+
+    if (!self->sheet && GTK_IS_WINDOW_CONTROLS (self->start_controls))
+      return;
+
+    gtk_box_remove (GTK_BOX (self->start_box), self->start_controls);
+    self->start_controls = NULL;
+  }
+
   if (self->sheet)
     controls = adw_sheet_controls_new (GTK_PACK_START);
   else
@@ -263,10 +281,21 @@ create_start_controls (AdwHeaderBar *self)
 }
 
 static void
-create_end_controls (AdwHeaderBar *self)
+recreate_end_controls (AdwHeaderBar *self)
 {
   GtkWidget *controls;
 
+  if (self->end_controls) {
+    if (self->sheet && ADW_IS_SHEET_CONTROLS (self->end_controls))
+      return;
+
+    if (!self->sheet && GTK_IS_WINDOW_CONTROLS (self->end_controls))
+      return;
+
+    gtk_box_remove (GTK_BOX (self->end_box), self->end_controls);
+    self->end_controls = NULL;
+  }
+
   if (self->sheet)
     controls = adw_sheet_controls_new (GTK_PACK_END);
   else
@@ -280,8 +309,6 @@ create_end_controls (AdwHeaderBar *self)
                             self->end_box);
   gtk_box_append (GTK_BOX (self->end_box), controls);
   self->end_controls = controls;
-
-  update_decoration_layout (self, FALSE, TRUE);
 }
 
 static void
@@ -328,11 +355,8 @@ update_start_title_buttons (AdwHeaderBar *self)
     }
   }
 
-  if ((self->start_controls != NULL) == show)
-    return;
-
   if (show) {
-    create_start_controls (self);
+    recreate_start_controls (self);
   } else if (self->start_box && self->start_controls) {
     gtk_box_remove (GTK_BOX (self->start_box), self->start_controls);
     self->start_controls = NULL;
@@ -370,11 +394,8 @@ update_end_title_buttons (AdwHeaderBar *self)
     }
   }
 
-  if ((self->end_controls != NULL) == show)
-    return;
-
   if (show) {
-    create_end_controls (self);
+    recreate_end_controls (self);
   } else if (self->end_box && self->end_controls) {
     gtk_box_remove (GTK_BOX (self->end_box), self->end_controls);
     self->end_controls = NULL;
@@ -398,6 +419,12 @@ update_title (AdwHeaderBar *self)
   if (!self->title_label)
     return;
 
+  if (ADW_IS_BOTTOM_SHEET (self->sheet) &&
+      adw_bottom_sheet_get_show_drag_handle (ADW_BOTTOM_SHEET (self->sheet))) {
+    gtk_label_set_text (GTK_LABEL (self->title_label), NULL);
+    return;
+  }
+
   if (self->title_navigation_page)
     title = adw_navigation_page_get_title (ADW_NAVIGATION_PAGE (self->title_navigation_page));
 
@@ -442,6 +469,33 @@ construct_title_label (AdwHeaderBar *self)
   update_title (self);
 }
 
+static GtkWidget *
+find_sheet (GtkWidget *widget)
+{
+  GtkWidget *sheet, *bin, *parent;
+
+  sheet = adw_widget_get_ancestor (widget, ADW_TYPE_BOTTOM_SHEET, TRUE, FALSE);
+
+  if (!sheet)
+    sheet = adw_widget_get_ancestor (widget, ADW_TYPE_FLOATING_SHEET, TRUE, FALSE);
+
+  if (ADW_IS_BOTTOM_SHEET (sheet))
+    bin = adw_bottom_sheet_get_sheet_bin (ADW_BOTTOM_SHEET (sheet));
+  else if (ADW_IS_FLOATING_SHEET (sheet))
+    bin = adw_floating_sheet_get_sheet_bin (ADW_FLOATING_SHEET (sheet));
+  else
+    return NULL;
+
+  if (bin && (widget == bin || gtk_widget_is_ancestor (widget, bin)))
+    return sheet;
+
+  parent = gtk_widget_get_parent (sheet);
+  if (parent)
+    return find_sheet (parent);
+
+  return NULL;
+}
+
 static void
 adw_header_bar_root (GtkWidget *widget)
 {
@@ -454,10 +508,12 @@ adw_header_bar_root (GtkWidget *widget)
     adw_widget_get_ancestor (widget, ADW_TYPE_NAVIGATION_PAGE, TRUE, TRUE);
 
   self->dialog = adw_widget_get_ancestor (widget, ADW_TYPE_DIALOG, TRUE, FALSE);
+  self->sheet = find_sheet (widget);
 
-  self->sheet = adw_widget_get_ancestor (widget, ADW_TYPE_BOTTOM_SHEET, TRUE, FALSE);
-  if (!self->sheet)
-    self->sheet = adw_widget_get_ancestor (widget, ADW_TYPE_FLOATING_SHEET, TRUE, FALSE);
+  if (ADW_IS_BOTTOM_SHEET (self->sheet)) {
+    g_signal_connect_swapped (self->sheet, "notify::show-drag-handle",
+                              G_CALLBACK (update_title), widget);
+  }
 
   if (self->title_navigation_page) {
     g_signal_connect_swapped (self->title_navigation_page, "notify::title",
@@ -548,9 +604,14 @@ adw_header_bar_unroot (GtkWidget *widget)
                                           update_title, widget);
   }
 
+  if (self->sheet) {
+    g_signal_handlers_disconnect_by_func (self->sheet,
+                                          update_title, widget);
+  }
+
   self->title_navigation_page = NULL;
   self->dialog = NULL;
-  self->sheet = NULL;
+  self->sheet = FALSE;
 
   for (l = self->split_views; l; l = l->next) {
     SplitViewData *data = l->data;
@@ -563,8 +624,6 @@ adw_header_bar_unroot (GtkWidget *widget)
 
   g_clear_pointer (&self->split_views, g_slist_free);
 
-  update_title_buttons (self);
-
   GTK_WIDGET_CLASS (adw_header_bar_parent_class)->unroot (widget);
 }
 
@@ -685,7 +744,7 @@ adw_header_bar_class_init (AdwHeaderBarClass *class)
   widget_class->compute_expand = adw_widget_compute_expand_horizontal_only;
 
   /**
-   * AdwHeaderBar:title-widget: (attributes org.gtk.Property.get=adw_header_bar_get_title_widget org.gtk.Property.set=adw_header_bar_set_title_widget)
+   * AdwHeaderBar:title-widget:
    *
    * The title widget to display.
    *
@@ -710,7 +769,7 @@ adw_header_bar_class_init (AdwHeaderBarClass *class)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwHeaderBar:show-start-title-buttons: (attributes org.gtk.Property.get=adw_header_bar_get_show_start_title_buttons org.gtk.Property.set=adw_header_bar_set_show_start_title_buttons)
+   * AdwHeaderBar:show-start-title-buttons:
    *
    * Whether to show title buttons at the start of the header bar.
    *
@@ -727,7 +786,7 @@ adw_header_bar_class_init (AdwHeaderBarClass *class)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwHeaderBar:show-end-title-buttons: (attributes org.gtk.Property.get=adw_header_bar_get_show_end_title_buttons org.gtk.Property.set=adw_header_bar_set_show_end_title_buttons)
+   * AdwHeaderBar:show-end-title-buttons:
    *
    * Whether to show title buttons at the end of the header bar.
    *
@@ -744,7 +803,7 @@ adw_header_bar_class_init (AdwHeaderBarClass *class)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwHeaderBar:show-back-button: (attributes org.gtk.Property.get=adw_header_bar_get_show_back_button org.gtk.Property.set=adw_header_bar_set_show_back_button)
+   * AdwHeaderBar:show-back-button:
    *
    * Whether the header bar can show the back button.
    *
@@ -759,7 +818,7 @@ adw_header_bar_class_init (AdwHeaderBarClass *class)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwHeaderBar:decoration-layout: (attributes org.gtk.Property.get=adw_header_bar_get_decoration_layout org.gtk.Property.set=adw_header_bar_set_decoration_layout)
+   * AdwHeaderBar:decoration-layout:
    *
    * The decoration layout for buttons.
    *
@@ -780,7 +839,7 @@ adw_header_bar_class_init (AdwHeaderBarClass *class)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwHeaderBar:centering-policy: (attributes org.gtk.Property.get=adw_header_bar_get_centering_policy org.gtk.Property.set=adw_header_bar_set_centering_policy)
+   * AdwHeaderBar:centering-policy:
    *
    * The policy for aligning the center widget.
    */
@@ -791,7 +850,7 @@ adw_header_bar_class_init (AdwHeaderBarClass *class)
                        G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwHeaderBar:show-title: (attributes org.gtk.Property.get=adw_header_bar_get_show_title org.gtk.Property.set=adw_header_bar_set_show_title)
+   * AdwHeaderBar:show-title:
    *
    * Whether the title widget should be shown.
    *
@@ -978,7 +1037,7 @@ adw_header_bar_remove (AdwHeaderBar *self,
 }
 
 /**
- * adw_header_bar_get_title_widget: (attributes org.gtk.Method.get_property=title-widget)
+ * adw_header_bar_get_title_widget:
  * @self: a header bar
  *
  * Gets the title widget widget of @self.
@@ -994,7 +1053,7 @@ adw_header_bar_get_title_widget (AdwHeaderBar *self)
 }
 
 /**
- * adw_header_bar_set_title_widget: (attributes org.gtk.Method.set_property=title-widget)
+ * adw_header_bar_set_title_widget:
  * @self: a header bar
  * @title_widget: (nullable): a widget to use for a title
  *
@@ -1044,7 +1103,7 @@ adw_header_bar_set_title_widget (AdwHeaderBar *self,
 }
 
 /**
- * adw_header_bar_get_show_start_title_buttons: (attributes org.gtk.Method.get_property=show-start-title-buttons)
+ * adw_header_bar_get_show_start_title_buttons:
  * @self: a header bar
  *
  * Gets whether to show title buttons at the start of @self.
@@ -1060,7 +1119,7 @@ adw_header_bar_get_show_start_title_buttons (AdwHeaderBar *self)
 }
 
 /**
- * adw_header_bar_set_show_start_title_buttons: (attributes org.gtk.Method.set_property=show-start-title-buttons)
+ * adw_header_bar_set_show_start_title_buttons:
  * @self: a header bar
  * @setting: `TRUE` to show standard title buttons
  *
@@ -1092,7 +1151,7 @@ adw_header_bar_set_show_start_title_buttons (AdwHeaderBar *self,
 }
 
 /**
- * adw_header_bar_get_show_end_title_buttons: (attributes org.gtk.Method.get_property=show-end-title-buttons)
+ * adw_header_bar_get_show_end_title_buttons:
  * @self: a header bar
  *
  * Gets whether to show title buttons at the end of @self.
@@ -1108,7 +1167,7 @@ adw_header_bar_get_show_end_title_buttons (AdwHeaderBar *self)
 }
 
 /**
- * adw_header_bar_set_show_end_title_buttons: (attributes org.gtk.Method.set_property=show-end-title-buttons)
+ * adw_header_bar_set_show_end_title_buttons:
  * @self: a header bar
  * @setting: `TRUE` to show standard title buttons
  *
@@ -1140,7 +1199,7 @@ adw_header_bar_set_show_end_title_buttons (AdwHeaderBar *self,
 }
 
 /**
- * adw_header_bar_get_show_back_button: (attributes org.gtk.Method.get_property=show-back-button)
+ * adw_header_bar_get_show_back_button:
  * @self: a header bar
  *
  * Gets whether @self can show the back button.
@@ -1158,7 +1217,7 @@ adw_header_bar_get_show_back_button (AdwHeaderBar *self)
 }
 
 /**
- * adw_header_bar_set_show_back_button: (attributes org.gtk.Method.set_property=show-back-button)
+ * adw_header_bar_set_show_back_button:
  * @self: a header bar
  * @show_back_button: whether to show the back button
  *
@@ -1197,7 +1256,7 @@ adw_header_bar_set_show_back_button (AdwHeaderBar *self,
 }
 
 /**
- * adw_header_bar_get_decoration_layout: (attributes org.gtk.Method.get_property=decoration-layout)
+ * adw_header_bar_get_decoration_layout:
  * @self: a header bar
  *
  * Gets the decoration layout for @self.
@@ -1213,7 +1272,7 @@ adw_header_bar_get_decoration_layout (AdwHeaderBar *self)
 }
 
 /**
- * adw_header_bar_set_decoration_layout: (attributes org.gtk.Method.set_property=decoration-layout)
+ * adw_header_bar_set_decoration_layout:
  * @self: a header bar
  * @layout: (nullable): a decoration layout
  *
@@ -1245,7 +1304,7 @@ adw_header_bar_set_decoration_layout (AdwHeaderBar *self,
 }
 
 /**
- * adw_header_bar_get_centering_policy: (attributes org.gtk.Method.get_property=centering-policy)
+ * adw_header_bar_get_centering_policy:
  * @self: a header bar
  *
  * Gets the policy for aligning the center widget.
@@ -1261,7 +1320,7 @@ adw_header_bar_get_centering_policy (AdwHeaderBar *self)
 }
 
 /**
- * adw_header_bar_set_centering_policy: (attributes org.gtk.Method.set_property=centering-policy)
+ * adw_header_bar_set_centering_policy:
  * @self: a header bar
  * @centering_policy: the centering policy
  *
@@ -1290,7 +1349,7 @@ adw_header_bar_set_centering_policy (AdwHeaderBar       *self,
 }
 
 /**
- * adw_header_bar_get_show_title: (attributes org.gtk.Method.get_property=show-title)
+ * adw_header_bar_get_show_title:
  * @self: a header bar
  *
  * Gets whether the title widget should be shown.
@@ -1308,7 +1367,7 @@ adw_header_bar_get_show_title (AdwHeaderBar *self)
 }
 
 /**
- * adw_header_bar_set_show_title: (attributes org.gtk.Method.set_property=show-title)
+ * adw_header_bar_set_show_title:
  * @self: a header bar
  * @show_title: whether the title widget is visible
  *
