@@ -5,12 +5,12 @@
  */
 
 #include "config.h"
+#include <glib/gi18n.h>
 
 #include "adw-toast-overlay.h"
 
 #include "adw-animation-util.h"
 #include "adw-easing.h"
-#include "adw-macros-private.h"
 #include "adw-timed-animation.h"
 #include "adw-toast-private.h"
 #include "adw-toast-widget-private.h"
@@ -32,6 +32,8 @@
  *   <img src="toast-overlay.png" alt="toast-overlay">
  * </picture>
  *
+ * Much like [class@Gtk.Overlay], `AdwToastOverlay` is a container with a single
+ * main child, on top of which it can display a [class@Toast], overlaid.
  * Toasts can be shown with [method@ToastOverlay.add_toast].
  *
  * See [class@Toast] for details.
@@ -62,8 +64,6 @@
  * ## Accessibility
  *
  * `AdwToastOverlay` uses the `GTK_ACCESSIBLE_ROLE_TAB_GROUP` role.
- *
- * Since: 1.0
  */
 
 typedef struct {
@@ -75,6 +75,7 @@ typedef struct {
   AdwAnimation *show_animation;
   AdwAnimation *hide_animation;
 
+  gulong shown_id;
   gulong dismissed_id;
   gboolean postponing;
 } ToastInfo;
@@ -108,6 +109,8 @@ static GtkBuildableIface *parent_buildable_iface;
 static void
 free_toast_info (ToastInfo *info)
 {
+  if (info->shown_id && info->show_animation)
+    g_signal_handler_disconnect (info->show_animation, info->shown_id);
   if (info->dismissed_id && info->toast)
     g_signal_handler_disconnect (info->toast, info->dismissed_id);
 
@@ -238,6 +241,8 @@ show_toast (AdwToastOverlay *self,
             ToastInfo       *info)
 {
   AdwAnimationTarget *target;
+  const char *title, *button_label;
+  char *announcement;
 
   g_assert (!info->widget);
 
@@ -254,10 +259,31 @@ show_toast (AdwToastOverlay *self,
                              self->hiding_toasts ? REPLACE_DURATION : SHOW_DURATION,
                              target);
 
-  g_signal_connect_swapped (info->show_animation, "done",
-                            G_CALLBACK (show_done_cb), info);
+  info->shown_id = g_signal_connect_swapped (info->show_animation, "done",
+                                             G_CALLBACK (show_done_cb), info);
 
   adw_animation_play (info->show_animation);
+
+  title = adw_toast_get_title (info->toast);
+  button_label = adw_toast_get_button_label (info->toast);
+
+  if (title && button_label) {
+    announcement = g_strdup_printf (_("A toast appeared: %s, has a button: %s"),
+                                    title, button_label);
+  } else if (title && !button_label) {
+    announcement = g_strdup_printf (_("A toast appeared: %s"), title);
+  } else if (!title && button_label) {
+    announcement = g_strdup_printf (_("A toast appeared, has a button: %s"),
+                                    button_label);
+  } else {
+    announcement = g_strdup (_("A toast appeared"));
+  }
+
+  gtk_accessible_announce (GTK_ACCESSIBLE (self),
+                           announcement,
+                           GTK_ACCESSIBLE_ANNOUNCEMENT_PRIORITY_MEDIUM);
+
+  g_free (announcement);
 }
 
 static int
@@ -491,15 +517,14 @@ adw_toast_overlay_class_init (AdwToastOverlayClass *klass)
   object_class->set_property = adw_toast_overlay_set_property;
 
   widget_class->compute_expand = adw_widget_compute_expand;
+  widget_class->get_request_mode = adw_widget_get_request_mode;
   widget_class->measure = adw_toast_overlay_measure;
   widget_class->size_allocate = adw_toast_overlay_size_allocate;
 
   /**
-   * AdwToastOverlay:child: (attributes org.gtk.Property.get=adw_toast_overlay_get_child org.gtk.Property.set=adw_toast_overlay_set_child)
+   * AdwToastOverlay:child:
    *
    * The child widget.
-   *
-   * Since: 1.0
    */
   props[PROP_CHILD] =
     g_param_spec_object ("child", NULL, NULL,
@@ -552,8 +577,6 @@ adw_toast_overlay_buildable_init (GtkBuildableIface *iface)
  * Creates a new `AdwToastOverlay`.
  *
  * Returns: the new created `AdwToastOverlay`
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_toast_overlay_new (void)
@@ -562,14 +585,12 @@ adw_toast_overlay_new (void)
 }
 
 /**
- * adw_toast_overlay_get_child: (attributes org.gtk.Method.get_property=child)
+ * adw_toast_overlay_get_child:
  * @self: a toast overlay
  *
  * Gets the child widget of @self.
  *
  * Returns: (nullable) (transfer none): the child widget of @self
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_toast_overlay_get_child (AdwToastOverlay *self)
@@ -580,13 +601,11 @@ adw_toast_overlay_get_child (AdwToastOverlay *self)
 }
 
 /**
- * adw_toast_overlay_set_child: (attributes org.gtk.Method.set_property=child)
+ * adw_toast_overlay_set_child:
  * @self: a toast overlay
  * @child: (nullable): the child widget
  *
  * Sets the child widget of @self.
- *
- * Since: 1.0
  */
 void
 adw_toast_overlay_set_child (AdwToastOverlay *self,
@@ -595,6 +614,9 @@ adw_toast_overlay_set_child (AdwToastOverlay *self,
   g_return_if_fail (ADW_IS_TOAST_OVERLAY (self));
   g_return_if_fail (child == NULL || GTK_IS_WIDGET (child));
 
+  if (child)
+    g_return_if_fail (gtk_widget_get_parent (child) == NULL);
+
   if (self->child == child)
     return;
 
@@ -624,8 +646,6 @@ adw_toast_overlay_set_child (AdwToastOverlay *self,
  *
  * If called on a toast currently in the queue, the toast will be bumped
  * forward to be shown as soon as possible.
- *
- * Since: 1.0
  */
 void
 adw_toast_overlay_add_toast (AdwToastOverlay *self,
